__version__ = "0.1"
__author__ = "Redouane Hajjar <redouane.hajjar@chimie.uvsq.fr>"
__date__ = "Dec 2009"


from math import pow
from struct import unpack, pack
import os
import time

################################################################################
class FileReader:
    """
    write ASCII file from bruker FID file
    """
    #---------------------------------------------------------------------------
    def __init__(self, filepath):
        """
        """
        self.filepath = filepath
        self.realPart = []
        self.imaginaryPart = []
        
        # read fid
        try:
            file = open(os.path.join(self.filepath, "fid"), "rb")
        except:
            raise("failed to load")
            MSG("failed to load", "Error:") #For topspin
        self.data = file.read()
        file.close()
        
        # reads parameters
        self.bytorder = int(GETPAR("BYTORDA"))
        self.datasize = int(GETPAR("TDeff"))
        
        self.stars = "****************************************************"
        self.msg = "%s`\nBYTORDA = %d \nTDeff = %d  \n%s" \
        %(self.stars, self.bytorder, self.datasize, self.stars)
        
        if self.bytorder  == 1:
            self.readFidBE()
        elif  self.bytorder  == 0:
            self.readFidLE()
        
        #print self.msg
        MSG(self.msg, "FileReader") #For topspin
        
    #---------------------------------------------------------------------------
    def readFidBE(self):
        """
        reads bruker fid file
        utf-16-be (big endian)
        """
        self.fileout  = ""
        start = 0
        totalval = ""
        showval = ""

        # decode values
        self.msg += "\nId \t Real part \t Imaginary part"
        for index in range(self.datasize/2):
            valueRe = unpack( "i", self.data[start+(2*index)*4:4+\
                             (2*index)*4])[0]      # decode
            valueIm = unpack( "i", self.data[start+(2*index+1)*4:4+\
                             (2*index+1)*4])[0]    # decode
            totalval += "%d %d \n" %(valueRe, valueIm)
            showval  += "%d \t %d \t %d \n" %(index+1, valueRe, valueIm)
        self.msg += "\n %s"  %showval
        
        # write values
        self.writeFile(totalval)
        
    #---------------------------------------------------------------------------
    def readFidLE(self):
        """
        reads bruker fid file
        utf-16-le (little endian)
        """
        self.fileout  = ""
        start = 0
        totalval = ""
        showval = ""
        
        # decode values
        self.msg += "\nId \t Real part \t Imaginary part"
        for index in range(self.datasize/2):
            # reverse (ACDC = CDCA) and decode
            valueRe = unpack( "i", self.data[start+(2*index)*4:4+\
                             (2*index)*4][::-1] )[0] 
            # reverse (ACDC = CDCA) and decode
            valueIm = unpack( "i", self.data[start+(2*index+1)*4:4+\
                             (2*index+1)*4][::-1] )[0]
            totalval += "%d %d \n" %(valueRe, valueIm)
            showval  += "%d \t %d \t %d \n" %(index+1, valueRe, valueIm)
        self.msg += "\n %s"  %showval
        
        # write values
        self.writeFile(totalval)
            
    #---------------------------------------------------------------------------
    def writeFile(self, value):
        """
        writes FID value in an ASCII file
        """
        self.fileout = os.path.join(self.filepath, "noisy.txt")
        fileoutW = open(self.fileout, "w")
        fileoutW.write(value)
        fileoutW.close()
        
    #---------------------------------------------------------------------------
    def report(self):
        """
        """
        return self.fileout

################################################################################
result = DATASET_DIALOG("SVD denoising FID", CURDATA())
if (int(GETPAR("TDeff"))) == 0:
    MSG("Provide a nonzero TDeff value", "FileReader")
filepath = result[3] + "/data/" + result[4] + "/nmr/" + result[0] + "/" + result[1]
fread = FileReader(filepath)
print fread.report()
################################################################################
